library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
 
-- Only XST supports RAM inference
-- Infers Dual Port Block Ram 
 
 entity dpblockram is 
 port (
 	-- wishbone signals
   clk_i  : in std_logic; 
	stb_i: in std_logic;
 	we_i : in std_logic; 
 	adr_i: in std_logic_vector(4 downto 0); 
 	dat_i: in std_logic_vector(15 downto 0); 
	ack_o: out std_logic;
 	dat_o: out std_logic_vector(15 downto 0);
   -- non wishbone signals
	dpra : in std_logic_vector(4 downto 0); 
 	dpo  : out std_logic_vector(15 downto 0)); 
 end dpblockram; 
 
 architecture syn of dpblockram is 
 
 type ram_type is array (31 downto 0) of std_logic_vector (15 downto 0); 
 signal RAM : ram_type; 
 signal read_a : std_logic_vector(4 downto 0); 
 signal read_dpra : std_logic_vector(4 downto 0); 
 
 begin 
 process (clk_i) 
 begin 
 	if (clk_i'event and clk_i = '1') then  
 		if (we_i = '1' and stb_i = '1') then 
 			RAM(conv_integer(adr_i)) <= dat_i; 
 		end if; 
 		read_a <= adr_i; 
 		read_dpra <= dpra; 
 	end if; 
 end process; 
 
 dat_o <= RAM(conv_integer(read_a));
 ack_o <= stb_i;
 dpo <= RAM(conv_integer(read_dpra)); 
 
 end syn;
 

