library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity wbd2io is
    Port ( 
	 		  -- wishbone signals
	 		  clk_i : in std_logic;
           stb_i : in std_logic;
           we_i  : in std_logic;
			  adr_i : in std_logic_vector(3 downto 0);
           dat_i : in std_logic_vector(15 downto 0);
           dat_o : out std_logic_vector(15 downto 0);
           ack_o : out std_logic;

			  -- outputs to/from the D2 IO board
			   IO_clk : out std_logic;									--output clock (divided)
				cs : out std_logic;										--chip select
           	we : out std_logic;										--write enable
           	oe : out std_logic;										--output enable
 				addr : out std_logic_vector(5 downto 0);			--address bus for communications with CPLD
           	data : inout std_logic_vector(7 downto 0)			--bidir data bus between CPLD & design
			  );
end wbd2io;

architecture Behavioral of wbd2io is

component d2IO_driver is
	generic( AWidth : integer := 6;
				DWidth : integer := 8);
    Port ( 
	 			clk : in std_logic;										--sytem clock (50MHz Xtal)
				IO_clk: out std_logic;									--output clock (divided)
	         
				pb_reg : out std_logic_vector (14 downto 0); 	--register for external pushbutton reads
				sw_reg : out std_logic_vector (7 downto 0);		--register for external 2pos switches
				led_signal : in std_logic_vector (15 downto 0);	--register to write to LEDs
				sseg_signal : in std_logic_vector (15 downto 0);--register to write to 7-Segment Display	
			
				-- outputs to the D2 IO board
				cs : out std_logic;										--chip select
           	we : out std_logic;										--write enable
           	oe : out std_logic;										--output enable
 				addr : out std_logic_vector(5 downto 0);			--address bus for communications with CPLD
           	data : inout std_logic_vector(7 downto 0)			--bidir data bus between CPLD & design
			);
end component;

signal pb_reg : std_logic_vector (14 downto 0); 	--register for external pushbutton reads
signal sw_reg : std_logic_vector (7 downto 0);		--register for external 2pos switches
signal led_signal : std_logic_vector (15 downto 0);	--register to write to LEDs
signal sseg_signal : std_logic_vector (15 downto 0);--register to write to 7-Segment Display	

--signal sel_pb_reg_l, sel_pb_reg_u, sel_sw_reg, sel_none : std_logic;			

begin

	d2io_driver_unit : d2IO_driver
	port map ( 
	 			clk => clk_i, 	--sytem clock (50MHz Xtal)
				         
				pb_reg => pb_reg, 	      --register for external pushbutton reads
				sw_reg => sw_reg, 		   --register for external 2pos switches
				led_signal => led_signal,  --register to write to LEDs
				sseg_signal => sseg_signal,--register to write to 7-Segment Display	
			
				-- outputs to the D2 IO board
				IO_clk => IO_clk, --output clock (divided)
				cs => cs,    	--chip select
           	we => we,   	--write enable
           	oe => oe,   	--output enable
 				addr => addr,	--address bus for communications with CPLD
           	data => data	--bidir data bus between CPLD & design
			);

	process (clk_i)
	begin
		if( clk_i'event and clk_i = '1') then
			case (adr_i(2 downto 0)) is
				when "000" => dat_o <= '0' & pb_reg(14 downto 0);
				when "001" => dat_o <= "00000000" & sw_reg;
				when "010" => 
					if( we_i = '1' and stb_i = '1') then
						led_signal <= dat_i;
					end if;
					dat_o <= led_signal;
				when "011" => 
					if( we_i = '1' and stb_i = '1' ) then
						sseg_signal <= dat_i;
					end if;
					dat_o <= sseg_signal;
				when others => dat_o <= "0001001000110100"; --1234
		  	end case;
		end if;
	end process;

	ack_o <= stb_i;
end Behavioral;
